@@ -25,7 +25,9 @@ namespace
 constexpr unsigned int MaxStripeHeight = 3072;
 // Precision for the scaler blocks
 constexpr unsigned int ScalePrecision = 12;
-constexpr unsigned int UnityScale = 1 << 12;
+constexpr unsigned int PhasePrecision = 12;
+constexpr unsigned int UnityScale = 1 << ScalePrecision;
+constexpr unsigned int UnityPhase = 1 << PhasePrecision;
 // PPF properties
 constexpr unsigned int ResamplePrecision = 10;
 constexpr unsigned int NumPhases = 16;
@@ -36,7 +38,7 @@ void check_stride(pisp_image_format_config const &config)
 	if (config.stride % PISP_BACK_END_OUTPUT_MIN_ALIGN || config.stride2 % PISP_BACK_END_OUTPUT_MIN_ALIGN)
 		throw std::runtime_error("Output stride values not sufficiently aligned");
 
-	if (PISP_IMAGE_FORMAT_wallpaper(config.format) && (config.stride % 128 || config.stride2 % 128))
+	if (PISP_IMAGE_FORMAT_WALLPAPER(config.format) && (config.stride % 128 || config.stride2 % 128))
 		throw std::runtime_error("Wallpaper format should have 128-byte aligned rolls");
 
 	pisp_image_format_config check = config;
@@ -66,13 +68,13 @@ void finalise_inputs(pisp_be_config &config)
 	}
 	else if (config.global.rgb_enables & PISP_BE_RGB_ENABLE_INPUT)
 	{
-		if (PISP_IMAGE_FORMAT_sampling_420(config.input_format.format) && (config.input_format.width & 1))
+		if (PISP_IMAGE_FORMAT_SAMPLING_420(config.input_format.format) && (config.input_format.width & 1))
 			throw std::runtime_error("finalise_inputs: 420 input height must be even");
-		else if ((PISP_IMAGE_FORMAT_sampling_420(config.input_format.format) ||
-				  PISP_IMAGE_FORMAT_sampling_422(config.input_format.format)) &&
+		else if ((PISP_IMAGE_FORMAT_SAMPLING_420(config.input_format.format) ||
+				  PISP_IMAGE_FORMAT_SAMPLING_422(config.input_format.format)) &&
 				 (config.input_format.width & 1))
 			throw std::runtime_error("finalise_inputs: 420/422 input width must be even");
-		if (PISP_IMAGE_FORMAT_wallpaper(config.input_format.format))
+		if (PISP_IMAGE_FORMAT_WALLPAPER(config.input_format.format))
 		{
 			if ((config.input_format.stride & 127) || (config.input_format.stride2 & 127))
 				throw std::runtime_error("finalise_inputs: wallpaper format strides must be at least 128-byte aligned");
@@ -150,13 +152,13 @@ void finalise_decompression(pisp_be_config const &be_config)
 {
 	uint32_t fmt = be_config.input_format.format, bayer_enables = be_config.global.bayer_enables;
 
-	if (PISP_IMAGE_FORMAT_compressed(fmt) && !(bayer_enables & PISP_BE_BAYER_ENABLE_DECOMPRESS))
+	if (PISP_IMAGE_FORMAT_COMPRESSED(fmt) && !(bayer_enables & PISP_BE_BAYER_ENABLE_DECOMPRESS))
 		throw std::runtime_error("BackEnd::finalise: input compressed but decompression not enabled");
 
-	if (!PISP_IMAGE_FORMAT_compressed(fmt) && (bayer_enables & PISP_BE_BAYER_ENABLE_DECOMPRESS))
+	if (!PISP_IMAGE_FORMAT_COMPRESSED(fmt) && (bayer_enables & PISP_BE_BAYER_ENABLE_DECOMPRESS))
 		throw std::runtime_error("BackEnd::finalise: input uncompressed but decompression enabled");
 
-	if ((bayer_enables & PISP_BE_BAYER_ENABLE_DECOMPRESS) && !PISP_IMAGE_FORMAT_bps_8(fmt))
+	if ((bayer_enables & PISP_BE_BAYER_ENABLE_DECOMPRESS) && !PISP_IMAGE_FORMAT_BPS_8(fmt))
 		throw std::runtime_error("BackEnd::finalise: compressed input is not 8bpp");
 }
 
@@ -189,13 +191,13 @@ void finalise_tdn(pisp_be_config &config)
 	if (tdn_enabled && !tdn_output_enabled)
 		throw std::runtime_error("BackEnd::finalise: TDN output not enabled when TDN enabled");
 
-	if (PISP_IMAGE_FORMAT_compressed(fmt) && !tdn_compress_enabled)
+	if (PISP_IMAGE_FORMAT_COMPRESSED(fmt) && !tdn_compress_enabled)
 		throw std::runtime_error("BackEnd::finalise: TDN output compressed but compression not enabled");
 
-	if (!PISP_IMAGE_FORMAT_compressed(fmt) && tdn_compress_enabled)
+	if (!PISP_IMAGE_FORMAT_COMPRESSED(fmt) && tdn_compress_enabled)
 		throw std::runtime_error("BackEnd::finalise: TDN output uncompressed but compression enabled");
 
-	if (tdn_compress_enabled && !PISP_IMAGE_FORMAT_bps_8(fmt))
+	if (tdn_compress_enabled && !PISP_IMAGE_FORMAT_BPS_8(fmt))
 		throw std::runtime_error("BackEnd::finalise: TDN output does not match compression mode");
 
 	if (tdn_output_enabled)
@@ -222,11 +224,11 @@ void finalise_tdn(pisp_be_config &config)
 		// Make the TDN input match the output if it's unset. Usually this will be the sensible thing to do.
 		if (config.tdn_input_format.width == 0 && config.tdn_input_format.height == 0)
 			config.tdn_input_format = config.tdn_output_format;
-		if (PISP_IMAGE_FORMAT_compressed(fmt) && !tdn_decompress_enabled)
+		if (PISP_IMAGE_FORMAT_COMPRESSED(fmt) && !tdn_decompress_enabled)
 			throw std::runtime_error("BackEnd::finalise: TDN input compressed but decompression not enabled");
-		if (!PISP_IMAGE_FORMAT_compressed(fmt) && tdn_decompress_enabled)
+		if (!PISP_IMAGE_FORMAT_COMPRESSED(fmt) && tdn_decompress_enabled)
 			throw std::runtime_error("BackEnd::finalise: TDN input uncompressed but decompression enabled");
-		if (tdn_compress_enabled && !PISP_IMAGE_FORMAT_bps_8(fmt))
+		if (tdn_compress_enabled && !PISP_IMAGE_FORMAT_BPS_8(fmt))
 			throw std::runtime_error("BackEnd::finalise: TDN output does not match compression mode");
 	}
 }
@@ -243,17 +245,17 @@ void finalise_stitch(pisp_be_config &config)
 
 	if (stitch_enabled != stitch_input_enabled)
 		throw std::runtime_error("BackEnd::finalise: stitch and stitch_input should be enabled/disabled together");
-	if (stitch_input_enabled && PISP_IMAGE_FORMAT_compressed(input_fmt) && !stitch_decompress_enabled)
+	if (stitch_input_enabled && PISP_IMAGE_FORMAT_COMPRESSED(input_fmt) && !stitch_decompress_enabled)
 		throw std::runtime_error("BackEnd::finalise: stitch output compressed but decompression not enabled");
-	if (stitch_input_enabled && !PISP_IMAGE_FORMAT_compressed(input_fmt) && stitch_decompress_enabled)
+	if (stitch_input_enabled && !PISP_IMAGE_FORMAT_COMPRESSED(input_fmt) && stitch_decompress_enabled)
 		throw std::runtime_error("BackEnd::finalise: stitch output uncompressed but decompression enabled");
-	if (stitch_output_enabled && PISP_IMAGE_FORMAT_compressed(output_fmt) && !stitch_compress_enabled)
+	if (stitch_output_enabled && PISP_IMAGE_FORMAT_COMPRESSED(output_fmt) && !stitch_compress_enabled)
 		throw std::runtime_error("BackEnd::finalise: stitch output compressed but compression not enabled");
-	if (stitch_output_enabled && !PISP_IMAGE_FORMAT_compressed(output_fmt) && stitch_compress_enabled)
+	if (stitch_output_enabled && !PISP_IMAGE_FORMAT_COMPRESSED(output_fmt) && stitch_compress_enabled)
 		throw std::runtime_error("BackEnd::finalise: stitch output uncompressed but compression enabled");
-	if (stitch_decompress_enabled && !PISP_IMAGE_FORMAT_bps_8(input_fmt))
+	if (stitch_decompress_enabled && !PISP_IMAGE_FORMAT_BPS_8(input_fmt))
 		throw std::runtime_error("BackEnd::finalise: stitch input does not match compression mode");
-	if (stitch_compress_enabled && !PISP_IMAGE_FORMAT_bps_8(output_fmt))
+	if (stitch_compress_enabled && !PISP_IMAGE_FORMAT_BPS_8(output_fmt))
 		throw std::runtime_error("BackEnd::finalise: stitch output does not match compression mode");
 
 	if (stitch_output_enabled)
@@ -286,14 +288,14 @@ void finalise_output(pisp_be_output_format_config &config)
 	if (config.image.width < PISP_BACK_END_MIN_TILE_WIDTH || config.image.height < PISP_BACK_END_MIN_TILE_HEIGHT)
 		throw std::runtime_error("finalise_output: output image too small");
 
-	if (PISP_IMAGE_FORMAT_sampling_420(config.image.format) && (config.image.height & 1))
+	if (PISP_IMAGE_FORMAT_SAMPLING_420(config.image.format) && (config.image.height & 1))
 		throw std::runtime_error("finalise_output: 420 image height should be even");
 
-	if ((PISP_IMAGE_FORMAT_sampling_420(config.image.format) || PISP_IMAGE_FORMAT_sampling_422(config.image.format)) &&
+	if ((PISP_IMAGE_FORMAT_SAMPLING_420(config.image.format) || PISP_IMAGE_FORMAT_SAMPLING_422(config.image.format)) &&
 		(config.image.width & 1))
 		throw std::runtime_error("finalise_output: 420/422 image width should be even");
 
-	if (PISP_IMAGE_FORMAT_wallpaper(config.image.format))
+	if (PISP_IMAGE_FORMAT_WALLPAPER(config.image.format))
 	{
 		if ((config.image.stride & 127) || (config.image.stride2 & 127))
 			throw std::runtime_error("finalise_output: wallpaper image stride should be at least 128-byte aligned");
@@ -367,17 +369,17 @@ unsigned int get_pixel_alignment(uint32_t format, int byte_alignment)
 {
 	int alignment_pixels = byte_alignment; // for 8bpp formats
 
-	if (PISP_IMAGE_FORMAT_bps_16(format))
+	if (PISP_IMAGE_FORMAT_BPS_16(format))
 		alignment_pixels = byte_alignment / 2;
-	else if (PISP_IMAGE_FORMAT_bps_10(format))
+	else if (PISP_IMAGE_FORMAT_BPS_10(format))
 		alignment_pixels = byte_alignment * 3 / 4;
-	else if (PISP_IMAGE_FORMAT_bpp_32(format))
+	else if (PISP_IMAGE_FORMAT_BPP_32(format))
 		alignment_pixels = byte_alignment / 4;
 
-	if (PISP_IMAGE_FORMAT_planar(format) && !PISP_IMAGE_FORMAT_sampling_444(format))
+	if (PISP_IMAGE_FORMAT_PLANAR(format) && !PISP_IMAGE_FORMAT_SAMPLING_444(format))
 		alignment_pixels *= 2; // the UV planes in fully planar 420/422 output will have half the width
-	else if (PISP_IMAGE_FORMAT_interleaved(format) &&
-			 (PISP_IMAGE_FORMAT_sampling_422(format) || PISP_IMAGE_FORMAT_sampling_420(format)))
+	else if (PISP_IMAGE_FORMAT_INTERLEAVED(format) &&
+			 (PISP_IMAGE_FORMAT_SAMPLING_422(format) || PISP_IMAGE_FORMAT_SAMPLING_420(format)))
 		alignment_pixels /= 2; // YUYV type outputs need only 8 pixels to make 16 bytes
 
 	return alignment_pixels;
@@ -398,7 +400,7 @@ static tiling::Length2 calculate_input_alignment(pisp_be_config const &config)
 		PISP_LOG(debug, "RGB input enabled");
 		// Need 4 byte alignment AND even number of pixels. Height must be 2 row aligned only for 420 input.
 		return tiling::Length2(lcm(get_pixel_alignment(config.input_format.format, PISP_BACK_END_INPUT_ALIGN), 2),
-							   PISP_IMAGE_FORMAT_sampling_420(config.input_format.format) ? 2 : 1);
+							   PISP_IMAGE_FORMAT_SAMPLING_420(config.input_format.format) ? 2 : 1);
 	}
 
 	uint32_t bayer_enables = config.global.bayer_enables;
@@ -406,11 +408,11 @@ static tiling::Length2 calculate_input_alignment(pisp_be_config const &config)
 	int pixel_alignment = get_pixel_alignment(config.input_format.format, PISP_BACK_END_INPUT_ALIGN);
 
 	// If any input is compressed, we need 8 *pixel* alignment.
-	if (PISP_IMAGE_FORMAT_compressed(config.input_format.format) ||
+	if (PISP_IMAGE_FORMAT_COMPRESSED(config.input_format.format) ||
 		((bayer_enables & PISP_BE_BAYER_ENABLE_TDN_INPUT) &&
-		 PISP_IMAGE_FORMAT_compressed(config.tdn_input_format.format)) ||
+		 PISP_IMAGE_FORMAT_COMPRESSED(config.tdn_input_format.format)) ||
 		((bayer_enables & PISP_BE_BAYER_ENABLE_STITCH_INPUT) &&
-		 PISP_IMAGE_FORMAT_compressed(config.stitch_input_format.format)))
+		 PISP_IMAGE_FORMAT_COMPRESSED(config.stitch_input_format.format)))
 		pixel_alignment = lcm(pixel_alignment, PISP_BACK_END_COMPRESSED_ALIGN);
 
 	// If any of the Bayer outputs are enabled, those need 16 *byte* alignment. (This already covers the outputs being compressed.)
@@ -426,7 +428,7 @@ static tiling::Length2 calculate_input_alignment(pisp_be_config const &config)
 
 static tiling::Length2 calculate_output_alignment(uint32_t format, int align = PISP_BACK_END_OUTPUT_MAX_ALIGN)
 {
-	int y_alignment = PISP_IMAGE_FORMAT_sampling_420(format) ? 2 : 1;
+	int y_alignment = PISP_IMAGE_FORMAT_SAMPLING_420(format) ? 2 : 1;
 
 	return tiling::Length2(get_pixel_alignment(format, align), y_alignment);
 }
@@ -488,9 +490,9 @@ void BackEnd::finaliseConfig()
 		if (enabled)
 		{
 			// crop is enabled when it contains non-zero width/height
-			uint16_t w = be_config_extra_.crop.width ? be_config_extra_.crop.width
+			uint16_t w = be_config_extra_.crop[j].width ? be_config_extra_.crop[j].width
 														: be_config_.input_format.width;
-			uint16_t h = be_config_extra_.crop.width ? be_config_extra_.crop.height
+			uint16_t h = be_config_extra_.crop[j].width ? be_config_extra_.crop[j].height
 														: be_config_.input_format.height;
 
 			if (dirty_flags_rgb & PISP_BE_RGB_ENABLE_DOWNSCALE(j))
@@ -537,18 +539,18 @@ void BackEnd::updateSmartResize()
 {
 	std::string filter;
 
-	// First get the size of the input to the rescalers. The crops are zero when not in use.
-	uint16_t input_width = be_config_extra_.crop.width;
-	if (!input_width)
-		input_width = be_config_.input_format.width;
-	uint16_t input_height = be_config_extra_.crop.height;
-	if (!input_height)
-		input_height = be_config_.input_format.height;
-
 	// Look through the output branches adjusting the scaling blocks where "smart resizing"
 	// has been requested.
 	for (unsigned int i = 0; i < variant_.BackEndNumBranches(0); i++)
 	{
+		// First get the size of the input to the rescalers. The crops are zero when not in use.
+		uint16_t input_width = be_config_extra_.crop[i].width;
+		if (!input_width)
+			input_width = be_config_.input_format.width;
+		uint16_t input_height = be_config_extra_.crop[i].height;
+		if (!input_height)
+			input_height = be_config_.input_format.height;
+
 		if ((smart_resize_dirty_ & (1 << i)) || (be_config_extra_.dirty_flags_extra & PISP_BE_DIRTY_CROP))
 		{
 			if (smart_resize_[i].width && smart_resize_[i].height)
@@ -691,15 +693,16 @@ void BackEnd::updateTiles()
 		PISP_LOG(debug, "Input alignments are " << tiling_config.input_alignment << " pixels");
 
 		tiling_config.input_image_size = tiling::Length2(c.input_format.width, c.input_format.height);
-		tiling_config.crop = tiling::Interval2(tiling::Interval(ce.crop.offset_x, ce.crop.width),
-											   tiling::Interval(ce.crop.offset_y, ce.crop.height));
-
-		if (tiling_config.crop.x.length == 0 || tiling_config.crop.y.length == 0)
-			tiling_config.crop = tiling::Interval2(tiling::Interval(0, c.input_format.width),
-												   tiling::Interval(0, c.input_format.height));
 
 		for (unsigned int i = 0; i < variant_.BackEndNumBranches(0); i++)
 		{
+			tiling_config.crop[i] = tiling::Interval2(tiling::Interval(ce.crop[i].offset_x, ce.crop[i].width),
+													  tiling::Interval(ce.crop[i].offset_y, ce.crop[i].height));
+
+			if (tiling_config.crop[i].x.length == 0 || tiling_config.crop[i].y.length == 0)
+				tiling_config.crop[i] = tiling::Interval2(tiling::Interval(0, c.input_format.width),
+														  tiling::Interval(0, c.input_format.height));
+
 			tiling_config.output_h_mirror[i] = be_config_.output_format[i].transform & PISP_BE_TRANSFORM_HFLIP;
 			tiling_config.downscale_factor[i] =
 				tiling::Length2(c.downscale[i].scale_factor_h, c.downscale[i].scale_factor_v);
@@ -794,7 +797,7 @@ std::vector<pisp_tile> BackEnd::retilePipeline(TilingConfig const &tiling_config
 			}
 
 			tiling::Crop2 downscale_crop;
-			tiling::Interval2 resample_size = tiles[i].crop.output;
+			tiling::Interval2 resample_size = tiles[i].crop[j].output;
 			resample_size.x = resample_size.x - tiles[i].resample[j].crop.x;
 			resample_size.y = resample_size.y - tiles[i].resample[j].crop.y;
 
@@ -802,17 +805,17 @@ std::vector<pisp_tile> BackEnd::retilePipeline(TilingConfig const &tiling_config
 			// next block. Also there will be no extra crop necessary for the resize operation.
 			if (be_config_.global.rgb_enables & PISP_BE_RGB_ENABLE_DOWNSCALE(j))
 			{
-				downscale_crop = tiles[i].downscale[j].crop + tiles[i].crop.crop;
+				downscale_crop = tiles[i].downscale[j].crop + tiles[i].crop[j].crop;
 				// Size of the tile going into the resample block needs to be set here.
 				resample_size = tiles[i].downscale[j].output;
 			}
 			else if (be_config_.global.rgb_enables & PISP_BE_RGB_ENABLE_RESAMPLE(j))
 			{
-				downscale_crop = tiles[i].resample[j].crop + tiles[i].crop.crop;
+				downscale_crop = tiles[i].resample[j].crop + tiles[i].crop[j].crop;
 			}
 			else
 			{
-				downscale_crop = tiles[i].output[j].crop + tiles[i].crop.crop;
+				downscale_crop = tiles[i].output[j].crop + tiles[i].crop[j].crop;
 			}
 
 			t.crop_x_start[j] = downscale_crop.x.start;
@@ -836,8 +839,8 @@ std::vector<pisp_tile> BackEnd::retilePipeline(TilingConfig const &tiling_config
 					unsigned int frac_y = (resample_size.y.offset * be_config_.downscale[j].scale_factor_v) &
 										  ((1 << ScalePrecision) - 1);
 					// Fractional component of the input required to generate the output pixel.
-					t.downscale_phase_x[p * variant_.BackEndNumBranches(0) + j] = (UnityScale - frac_x);
-					t.downscale_phase_y[p * variant_.BackEndNumBranches(0) + j] = (UnityScale - frac_y);
+					t.downscale_phase_x[p * variant_.BackEndNumBranches(0) + j] = (UnityPhase - frac_x);
+					t.downscale_phase_y[p * variant_.BackEndNumBranches(0) + j] = (UnityPhase - frac_y);
 				}
 
 				if (be_config_.global.rgb_enables & PISP_BE_RGB_ENABLE_RESAMPLE(j))
@@ -858,15 +861,15 @@ std::vector<pisp_tile> BackEnd::retilePipeline(TilingConfig const &tiling_config
 					t.resample_phase_y[p * variant_.BackEndNumBranches(0) + j] +=
 						be_config_extra_.resample[j].initial_phase_v[p];
 					// Have to be within this range, else some calculation went wrong.
-					PISP_ASSERT(t.resample_phase_x[p * variant_.BackEndNumBranches(0) + j] <= 2 * (UnityScale - 1));
-					PISP_ASSERT(t.resample_phase_y[p * variant_.BackEndNumBranches(0) + j] <= 2 * (UnityScale - 1));
+					PISP_ASSERT(t.resample_phase_x[p * variant_.BackEndNumBranches(0) + j] <= (2 * UnityPhase - 1));
+					PISP_ASSERT(t.resample_phase_y[p * variant_.BackEndNumBranches(0) + j] <= (2 * UnityPhase - 1));
 				}
 			}
 
 			// Phase difference between planes cannot be > 0.5 pixels on the output dimenstions.
 			if (be_config_.global.rgb_enables & PISP_BE_RGB_ENABLE_RESAMPLE(j))
 			{
-				int phase_max = (be_config_.resample[j].scale_factor_h * UnityScale / 2) >> ScalePrecision;
+				int phase_max = (be_config_.resample[j].scale_factor_h * UnityPhase / 2) >> ScalePrecision;
 				if (std::abs(t.resample_phase_x[0 * variant_.BackEndNumBranches(0) + j] -
 							 t.resample_phase_x[1 * variant_.BackEndNumBranches(0) + j]) > phase_max ||
 					std::abs(t.resample_phase_x[1 * variant_.BackEndNumBranches(0) + j] -
@@ -876,7 +879,7 @@ std::vector<pisp_tile> BackEnd::retilePipeline(TilingConfig const &tiling_config
 				{
 					throw std::runtime_error("Resample phase x for tile is > 0.5 pixels on the output dimensions.");
 				}
-				phase_max = (be_config_.resample[j].scale_factor_v * UnityScale / 2) >> ScalePrecision;
+				phase_max = (be_config_.resample[j].scale_factor_v * UnityPhase / 2) >> ScalePrecision;
 				if (std::abs(t.resample_phase_y[0 * variant_.BackEndNumBranches(0) + j] -
 							 t.resample_phase_y[1 * variant_.BackEndNumBranches(0) + j]) > phase_max ||
 					std::abs(t.resample_phase_y[1 * variant_.BackEndNumBranches(0) + j] -
@@ -952,8 +955,8 @@ void BackEnd::getOutputSize(int i, uint16_t *width, uint16_t *height, pisp_image
 		*width = be_config_extra_.resample[i].scaled_width, *height = be_config_extra_.resample[i].scaled_height;
 	else if (be_config_.global.rgb_enables & PISP_BE_RGB_ENABLE_DOWNSCALE(i))
 		*width = be_config_extra_.downscale[i].scaled_width, *height = be_config_extra_.downscale[i].scaled_height;
-	else if (be_config_extra_.crop.width) // crop width and height will be zero when crop disabled
-		*width = be_config_extra_.crop.width, *height = be_config_extra_.crop.height;
+	else if (be_config_extra_.crop[i].width) // crop width and height will be zero when crop disabled
+		*width = be_config_extra_.crop[i].width, *height = be_config_extra_.crop[i].height;
 	else
 		*width = ifmt.width, *height = ifmt.height;
 }
